@@ -40,22 +40,59 @@
 #include <sys/time.h>
 #endif
 #endif
+#ifndef HAVE_GETTIMEOFDAY
+#include "gettimeofday.h"
+#endif
 
 #include "htp.h"
 
 static struct timeserver_st timeservers[128];
 static unsigned int timeind = 0;
 char *http_proxy = NULL;
-uint16_t http_proxy_port = 0;
+unsigned int http_proxy_port = 0;
 
-static void daemonize(void) {
+#ifdef _USE_WIN32_
+static SC_HANDLE manager = NULL;
+static SC_HANDLE service = NULL;
+static SERVICE_STATUS htpServiceStat;
+static SERVICE_STATUS_HANDLE htpServiceStat_handle;
+
+void WINAPI htpServiceStart(DWORD argc, LPTSTR *argv) {
+	htpServiceStat.dwServiceType = SERVICE_WIN32;
+	htpServiceStat.dwCurrentState = SERVICE_START_PENDING;
+	htpServiceStat.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+	htpServiceStat.dwWin32ExitCode = 0;
+	htpServiceStat.dwServiceSpecificExitCode = 0;
+	htpServiceStat.dwCheckPoint = 0;
+	htpServiceStat.dwWaitHint = 0;
+
+	htpServiceStat_handle = RegisterServiceCtrlHandler("htpd", NULL); 
+
+	if (htpServiceStat_handle == (SERVICE_STATUS_HANDLE) 0) {
+		return;
+	}
+
+	htpServiceStat.dwCurrentState = SERVICE_RUNNING;
+	htpServiceStat.dwCheckPoint = 0;
+	htpServiceStat.dwWaitHint = 0;
+	SetServiceStatus(htpServiceStat_handle, &htpServiceStat);
+
+	return;
+}
+#endif
+
+static void daemonize(int argc, char **argv) {
+#ifndef _USE_WIN32_
+#ifdef HAVE_FORK
 	pid_t pid;
 
 	close(STDIN_FILENO);
 	close(STDOUT_FILENO);
 	close(STDERR_FILENO);
 
+#ifdef HAVE_SETSID
 	setsid();
+#endif
 
 	pid = fork();
 
@@ -66,7 +103,80 @@ static void daemonize(void) {
 	if (pid < 0) {
 		exit(EXIT_FAILURE);
 	}
+#endif
+#else
+	/* Create a Win32 Service. */
+	/* Most of this was taken from "tinc" (src/process.c) */
+	SERVICE_DESCRIPTION description = {"Hypertext Time Protocol Daemon (HTPD)"};
+	SERVICE_TABLE_ENTRY serviceTable[] = {
+		{"htpd", htpServiceStart},
+		{ NULL, NULL}
+	};
+	char svcCommand[8192], *svcptr;
+	int argind;
 
+	/*
+	 * This whole mess is to construct the "lpBinaryPathName" which needs
+	 * quotes and all kinds of other non-sense.
+	 */
+	svcptr = svcCommand;
+
+	strcpy(svcptr, "\"");
+	svcptr += strlen(svcptr);  /* This is not the same as svcptr++. */
+
+	/* If the full path wasn't specified, assume it's in the current directory. */
+	if (strchr(argv[0], '\\') == NULL) {
+		GetCurrentDirectory(sizeof(svcCommand) - (svcptr - svcCommand) - 1, svcptr);
+		svcptr += strlen(svcptr);
+		strncat(svcptr, "\\", sizeof(svcCommand) - (svcptr - svcCommand) - 1);
+		svcptr += strlen(svcptr);
+	}
+	strncat(svcptr, argv[0], sizeof(svcCommand) - (svcptr - svcCommand) - 1);
+	svcptr += strlen(svcptr);
+	strncat(svcptr, "\"", sizeof(svcCommand) - (svcptr - svcCommand) - 1);
+	svcptr += strlen(svcptr);
+
+	for (argind = 1; argind < argc; argind++) {
+		strncat(svcptr, " \"", sizeof(svcCommand) - (svcptr - svcCommand) - 1);
+		svcptr += strlen(svcptr);
+
+		strncat(svcptr, argv[argind], sizeof(svcCommand) - (svcptr - svcCommand) - 1);
+		svcptr += strlen(svcptr);
+
+		strncat(svcptr, "\"", sizeof(svcCommand) - (svcptr - svcCommand) - 1);
+		svcptr += strlen(svcptr);
+	}
+
+	if (!StartServiceCtrlDispatcher(serviceTable)) {
+		fprintf(stderr, "Could not start service dispatcher.\n");
+		exit(EXIT_FAILURE);
+	}
+
+	manager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
+	if (manager == NULL) {
+		fprintf(stderr, "Could not contact service manager.\n");
+		exit(EXIT_FAILURE);
+	}
+
+	service = CreateService(manager, "htpd", "htpd", SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, SERVICE_AUTO_START, SERVICE_ERROR_NORMAL, svcCommand, "NDIS", NULL, NULL, NULL, NULL);
+
+	if (service == NULL) {
+		fprintf(stderr, "Could not create service.\n");
+		exit(EXIT_FAILURE);
+	}
+
+	ChangeServiceConfig2(service, SERVICE_CONFIG_DESCRIPTION, &description);
+
+	if (!StartService(service, 0, NULL)) {
+		fprintf(stderr, "Could not start service!\n");
+		exit(EXIT_FAILURE);
+	} else {
+		printf("Service started.\n");
+	}
+
+
+	exit(EXIT_SUCCESS);
+#endif
 	return;
 }
 
@@ -190,6 +300,7 @@ int add_server(const char *shortvar, const char *var, const char *arguments, con
 }
 
 int print_help(const char *shortvar, const char *var, const char *arguments, const char *value, lc_flags_t flags, void *extra) {
+	printf("htpd version " PACKAGE_VERSION "\n");
 	printf("Usage: htpd [-M max] [-m min] [-P proxy] [-H host [-H host [...]]]\n");
 	printf("   Where each `host' is in format of:\n");
 	printf("       hostname[:port]\n");
@@ -226,9 +337,11 @@ int main(int argc, char *argv[]) {
 		return(EXIT_FAILURE);
 	}
 
-	daemonize();
+	daemonize(argc, argv);
 
+#ifdef HAVE_OPENLOG
 	openlog("htpd", LOG_PID, LOG_DAEMON);
+#endif
 
 	while (1) {
 		newtime = htp_calctime(timeservers, totaltimeservers, http_proxy, http_proxy_port);
@@ -254,7 +367,9 @@ int main(int argc, char *argv[]) {
 			sleeptime = maxsleeptime;
 		}
 
+#ifdef HAVE_SYSLOG
 		syslog(LOG_INFO, "Sleeping for %li seconds.", sleeptime);
+#endif
 
 		sleep(sleeptime);
 	}
