@@ -18,6 +18,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_LIBCONFIG
+#include <libconfig.h>
+#endif
 #ifdef HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
@@ -31,6 +34,15 @@
 
 #include "htp.h"
 
+int print_help(const char *shortvar, const char *var, const char *arguments, const char *value, lc_flags_t flags, void *extra) {
+	printf("htpdate version " PACKAGE_VERSION "\n");
+	printf("Usage: htpdate [-ah] <host> [<host> [<host> [...]]]\n");
+	printf("  -a      Allow htpdate to skew the time instead of setting it.\n");
+	printf("  -h      This help information.\n");
+	printf("  host    Host to poll for time, in the format of hostname[:port]\n");
+	exit(EXIT_SUCCESS);
+}
+
 int main(int argc, char *argv[]) {
 	unsigned int argind = 0, timeind = 0;
 	unsigned int totaltimeservers = 0;
@@ -39,20 +51,25 @@ int main(int argc, char *argv[]) {
 	char *host = NULL, *portstr = NULL;
 	char *proxyhost = NULL, *proxyportstr = NULL;
 	int port = -1, proxyport = -1;
+	int allowadj = 0;
+	int lc_p_ret;
 
 	htp_init();
 
+#ifdef HAVE_OPENLOG
 	openlog("htpdate", LOG_PID, LOG_USER);
+#endif
 
-	if ( argc <= 1 ) {
-		printf("Usage: htpdate <host> [<host> [<host> [...]]]\n");
-		printf("  Where each `host' is in the format of:\n");
-		printf("       hostname[:port]\n");
-		return(EXIT_FAILURE);
-	}
+	lc_register_var("AdjustTime", LC_VAR_BOOL_BY_EXISTANCE, &allowadj, 'a');
+        lc_register_callback(NULL, 'h', LC_VAR_NONE, print_help, NULL);
+        lc_p_ret = lc_process(argc, argv, "htpdate", LC_CONF_SPACE, SYSCONFDIR "/htpdate.conf");
+        if (lc_p_ret < 0) {
+                fprintf(stderr, "Error processing configuration information: %s.\n", lc_geterrstr());
+                return(EXIT_FAILURE);
+        }
 
 	/* Process each argument as a host:port pair. */
-	for (argind = 1; argind < (unsigned int) argc; argind++) {
+	for (argind = lc_optind; argind < (unsigned int) argc; argind++) {
 		if (argind >= (sizeof(timeservers)/sizeof(timeservers[0]))) {
 			break;
 		}
@@ -111,7 +128,7 @@ int main(int argc, char *argv[]) {
 		return(EXIT_FAILURE);
 	}
 
-	set_clock(newtime, 0);
+	set_clock(newtime, allowadj);
 
 	printf("new time: %s", asctime(localtime(&newtime)));
 
